macroscript SpookyHide category:"Bobo_s Tools"
(

----------------------------------------------------------------------------------------
--   Script Name:	SpookyHide
--	 Version:		v0.31
--	 Started:       02/10/01
--	 Last Modified: 07/05/01
--   Description:	Hide/Freeze/SeeThru Objects based on other object's selection
--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4/spookyhide
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
--   Idea by:       Paul Neale
----------------------------------------------------------------------------------------

global SpookyHide_floater
local Triggers_array =#()
local Reactors_array =#()
persistent global spookyHide_linksArray

rollout SpookyHide_roll "Spooky Hide"
(
Group "Manage Sets"
(
button new_set "New" width:48 across:3 height:18 tooltip:"Create a NEW Set of Triggers/Reactors"
button rename_set "Rename" width:50 height:18 align:#center tooltip:"RENAME the current Set of Triggers/Reactors"
button delete_set "Delete" width:50 height:18 tooltip:"DELETE the current Set of Triggers/Reactors from the Scene"
edittext link_set_name width:155 align:#center offset:[-2,0]
checkbutton set_enabled "On" across:2 align:#left width:24 offset:[-3,0] highlightcolor:(color 200 255 200) tooltip:"TOGGLE the current Set of Triggers/Reactors On/Off"
dropdownlist link_set_list items:#() width:130 align:#right offset:[5,0]

button enable_all "All On" width:48 across:3 height:18 tooltip:"Turn ON all Sets of Triggers/Reactors in the scene"
button disable_all "All Off" width:50 height:18 align:#center tooltip:"Turn OFF all Sets of Triggers/Reactors in the scene"
button invert_state "Invert" width:50 height:18 tooltip:"Turn ON all Sets that are currently off and vice-versa"

)

Group "Triggers"
(
button get_trigger "Get Triggers" width:70 align:#left across:2 offset:[-3,0] tooltip:"Get the current Scene selection as Triggers"
button select_trigger "Select Triggers" width:80 align:#right offset:[3,0] tooltip:"Select the Triggers in the scene"
listbox triggers items:#() height:5 width:160 align:#center
)

Group "Reactors"
(
button get_reactors "Get Reactors" width:70 align:#left across:2 offset:[-3,0] tooltip:"Get the current Scene selection as Reactors"
button select_reactors "Select Reactors" width:80 align:#right offset:[3,0] tooltip:"Select the Reactors in the scene"
listbox reactors items:#() height:5 width:160 align:#center
)


dropdownlist set_reaction "If Trigger Selected/Deselected:" width:160 align:#center \
items:#("Hide/Unhide Reactors","Unhide/Hide Reactors","Freeze/Unfreeze Reactors", "Unfreeze/Freeze Reactors","SeeThru/Opaque Reactors","Opaque/SeeThru Reactors",\
"Box/Mesh Display","Mesh/Box Display","BackfaceCulling/No Culling","No Culling/BackfaceCulling","Select") 

fn gen_callbacks =
(
deleteallchangehandlers id:#hideUnhide
for s in spookyHide_linksArray do
(
if s[4] then
(
case s[5] of
(
1: action_txt=".isHidden = "
2: action_txt=".isHidden = NOT "
3: action_txt=".isFrozen = "
4: action_txt=".isFrozen = NOT "
5: action_txt=".xray = "
6: action_txt=".xray = NOT "
7: action_txt=".boxMode = "
8: action_txt=".boxMode = NOT "
9: action_txt=".backfaceCull = "
10: action_txt=".backfaceCull = NOT "
)

txt = "when select #("
for i = 1 to s[2].count do 
	(
	txt += "$'"+s[2][i]+"'"
	if i < 	s[2].count then txt +=", "
	)
txt += ") changes id:#hideUnhide obj do (\n" 

if s[5] < 11 then
(
for i in s[3] do 
	(
	txt += "try(if obj != $'"+i+"' then $'"+ i + "'"+action_txt+ " obj.isSelected)catch()\n"
	)
)
else
(
if s[5] == 11 then
	(
	txt += "sel_array = #()\n"
	txt += "disableSceneRedraw()\n"
	for i in s[3] do 
		(
		txt += "try(if obj != $'"+i+"' then append sel_array $'"+ i+ "')catch()\n"
		)
	txt += "select sel_array \n"
	txt += "enableSceneRedraw() \n"
	)
)

	
txt +=")"
execute txt
)--end if
)--end s loop
)--end fn

fn update_link_sets num =
(
temp_array = #()
for i in spookyHide_linksArray do
(
if i[4] == true then enabled_state = "+" else enabled_state = "--" 
append temp_array (enabled_state + i[1])
)

link_set_list.items = temp_array
set_enabled.checked = spookyHide_linksArray[num][4]
triggers.items = spookyHide_linksArray[num][2]
reactors.items = spookyHide_linksArray[num][3]
set_reaction.selection = spookyHide_linksArray[num][5]
delete_set.enabled = spookyHide_linksArray.count > 1 

gen_callbacks()
)--end fn

on enable_all pressed do
(
	for i in spookyHide_linksArray do
	(
		i[4] = true 
		update_link_sets link_set_list.selection
	)
)

on disable_all pressed do
(
	for i in spookyHide_linksArray do
	(
		i[4] = false
		update_link_sets link_set_list.selection
	)
)

on invert_state pressed do
(
	for i in spookyHide_linksArray do
	(
		i[4] = not i[4]
		update_link_sets link_set_list.selection
	)
)



on set_enabled changed state do
(
spookyHide_linksArray[link_set_list.selection][4] = state
update_link_sets link_set_list.selection
)

on set_reaction selected itm do
(
spookyHide_linksArray[link_set_list.selection][5] = itm
update_link_sets link_set_list.selection
)

on delete_set pressed do
(
q = querybox "Are You Sure You Want To DELETE The Set?" title:"SpookyHide"
if q then
	(
	deleteItem spookyHide_linksArray link_set_list.selection
	if link_set_list.selection == link_set_list.items.count then link_set_list.selection = link_set_list.items.count-1
	update_link_sets link_set_list.selection
	)
)



on get_trigger pressed do
(
if selection.count == 0 then
	(
	q = querybox "Selection Set is EMPTY!\nThis will remove any Triggers from the list.\nContinue?" title:"SpookyHide"
	)
	else
	(
	q = true
	)
	if q then
	(
	Triggers_array = selection as array
	temp_array = #()
	for i in Triggers_array do append temp_array i.name
	spookyHide_linksArray[link_set_list.selection][2] = temp_array
	update_link_sets link_set_list.selection
	)
)

on get_reactors pressed do
(
if selection.count == 0 then
	(
	q = querybox "Selection Set is EMPTY!\nThis will remove any Reactors from the list.\nContinue?" title:"SpookyHide"
	)
	else
	(
	q = true
	)
	if q then
	(
	Reactors_array = selection as array
	temp_array = #()
	for i in Reactors_array do append temp_array i.name
	spookyHide_linksArray[link_set_list.selection][3] = temp_array
	update_link_sets link_set_list.selection
	)
)

on select_trigger pressed do
(
temp_array = triggers.items 
sel_array = #()
for i in temp_array do 
	(
	try
	(
		obj = execute ("$'"+i+"'")
		if isValidNode obj then append sel_array obj
	)
	catch()
	)
select sel_array	
)

on select_reactors pressed do
(
temp_array = reactors.items 
sel_array = #()
for i in temp_array do 
	(
	try
	(
		obj = execute ("$'"+i+"'")
		if isValidNode obj then append sel_array obj
	)
	catch()
	)
select sel_array	
)

on triggers doubleclicked itm do
(
obj = execute ("$'"+triggers.selected+"'")
if isValidNode obj then select obj
)

on reactors doubleclicked itm do
(
obj = execute ("$'"+reactors.selected+"'")
if isValidNode obj then select obj
)

on new_set pressed do
(
if link_set_name.text.count == 0 then
	(
	link_set_name.text = "New Set "+(spookyHide_linksArray.count+1) as string
	)
new_array = #(link_set_name.text,#(),#(),true,1)	
append spookyHide_linksArray new_array 
update_link_sets spookyHide_linksArray.count
link_set_list.selection = spookyHide_linksArray.count
)

on link_set_list selected itm do
(
update_link_sets itm
link_set_name.text = spookyHide_linksArray[itm][1]
)

on rename_set pressed do
(
if link_set_name.text.count > 0 then
	(
	spookyHide_linksArray[link_set_list.selection][1] = link_set_name.text
	update_link_sets link_set_list.selection
	)
)

)--end rollout


try(closerolloutfloater SpookyHide_floater)catch()
SpookyHide_floater = newRolloutFloater "SpookyHide v0.31" 200 490 100 100
addrollout SpookyHide_roll SpookyHide_floater 
if spookyHide_linksArray == undefined or spookyHide_linksArray.count == 0 then
	(
	spookyHide_linksArray = #(#("New Set",#(),#(),true, 1))
	)
SpookyHide_roll.update_link_sets 1

	
)--end script